use futures::FutureExt;
use tokio::sync::mpsc;

use crate::message::Message;

#[derive(Clone, Debug, PartialEq)]
#[non_exhaustive]
pub enum ReceiverError {
    Empty,
    Closed,
}

impl From<mpsc::error::RecvError> for ReceiverError {
    fn from(_: mpsc::error::RecvError) -> Self {
        ReceiverError::Closed
    }
}

// impl From<mpsc::error::TryRecvError> for ReceiverError {
//     fn from(err: mpsc::error::TryRecvError) -> Self {
//         match err {
//             mpsc::error::TryRecvError::Empty => ReceiverError::Empty,
//             mpsc::error::TryRecvError::Closed => ReceiverError::Closed,
//             mpsc::error::TryRecvError::Lagged(_) => ReceiverError::Lagged,
//         }
//     }
// }

#[derive(Debug)]
pub struct Receiver<T> {
    receiver: mpsc::Receiver<T>,
}

impl<T: Message> Receiver<T> {
    pub(crate) fn new(receiver: mpsc::Receiver<T>) -> Self {
        Receiver { receiver }
    }

    pub async fn recv(&mut self) -> Result<T, ReceiverError> {
        self.receiver.recv().await.ok_or(ReceiverError::Closed)
    }

    pub fn recv_blocking(&mut self) -> Result<T, ReceiverError> {
        self.receiver.blocking_recv().ok_or(ReceiverError::Closed)
    }

    pub fn try_recv(&mut self) -> Result<T, ReceiverError> {
        // Ok(self.receiver.try_recv()?)
        self.recv()
            .now_or_never()
            .unwrap_or(Err(ReceiverError::Empty))
    }
}
