use tokio::sync::mpsc;

use crate::message::Message;

#[derive(Clone, Debug, PartialEq)]
#[non_exhaustive]
pub enum SenderError {
    Closed,
    Full
}

impl<T> From<mpsc::error::SendError<T>> for SenderError {
    fn from(_: mpsc::error::SendError<T>) -> Self {
        SenderError::Closed
    }
}

impl<T> From<mpsc::error::TrySendError<T>> for SenderError {
    fn from(err: mpsc::error::TrySendError<T>) -> Self {
        match err {
            mpsc::error::TrySendError::Full(_) => SenderError::Full,
            mpsc::error::TrySendError::Closed(_) => SenderError::Closed
        }
    }
}

#[derive(Clone, Debug)]
pub struct Sender<T> {
    sender: mpsc::Sender<T>,
}

impl<T: Message> Sender<T> {
    pub(crate) fn new(sender: mpsc::Sender<T>) -> Self {
        Sender { sender }
    }

    pub async fn send(&mut self, message: T) -> Result<(), SenderError> {
        self.sender.send(message).await?;
        Ok(())
    }

    pub fn send_blocking(&mut self, message: T) -> Result<(), SenderError> {
        self.sender.blocking_send(message)?;
        Ok(())
    }

    pub fn try_send(&mut self, message: T) -> Result<(), SenderError> {
        self.sender.try_send(message)?;
        Ok(())
    }
}
